#ifndef _RaveInterface_FlavorTagFactory_H_
#define _RaveInterface_FlavorTagFactory_H_

#include <rave/RaveTrack.h>
#include <rave/RaveVertex.h>
#include <rave/RaveCalibration.h>
#include <rave/VertexFactory.h>

class RaveReconstructor;

namespace rave
{
  class FlavorTagger;

/** 
 *  The class that is central to using the rave library.
 *  It builds the RaveVertices from a set of RaveTracks.
 *  \paramname method is a string that describes which
 *  algorithms should be employed and which parameter
 *  settings should be chosen.
 *  The default is chosen such that something
 *  "sensible" is done in most use cases.
 */

class FlavorTagFactory 
{
  public:
    FlavorTagFactory ( const RaveReconstructor & r );
    FlavorTagFactory ( const rave::VertexFactory & v );
    FlavorTagFactory ( const FlavorTagFactory & o );
    ~FlavorTagFactory();

    double tag ( const std::vector < rave::Track > & tracks, const rave::Vertex & primary,
                 const rave::Vector3D & jet ) const;
    void teach ( const std::vector < rave::Track > & tracks, const rave::Vertex & primary,
                 const rave::Vector3D & jet, char tag );

    bool hasFlavorTagging() const; //< has btagging code?
    /// calibrate the btagger
    void calibrate ( const rave::Calibration & ) const;
    /// get the calibration, optionally "finish the teacher"
    /// (=normalize) beforehand.
    rave::Calibration getCalibration() const;

  private:
    RaveReconstructor * theRector;
    mutable rave::FlavorTagger * theTagger;
};

}

#endif
