#include "RaveBase/RaveInterface/rave/RaveTrack.h"
#include "RaveBase/RaveEngine/interface/RaveId.h"

using namespace rave;

Track::Track( const Vector6D & s, const Covariance6D & e, Charge q,
   double chi2, double ndof, void * p, std::string d ) : 
     Base ( new BasicTrack ( s, e, q, chi2, ndof, p, d ) )
{}

Track::Track( int id, const Vector6D & s, const Covariance6D & e, Charge q,
   double chi2, double ndof, void * p, std::string d ) : 
     Base ( new BasicTrack ( id, s, e, q, chi2, ndof, p, d ) )
{}

Track::Track( const Vector6D & s, const Covariance6D & e, Charge q,
   double chi2, double ndof, int p, std::string d ) : 
  Base ( new BasicTrack ( s, e, q, chi2, ndof, (void *) (p), d ) )
{}

Track::Track ( const BasicTrack & o ) : Base ( o.clone() )
{}

Track::Track () : Base ( new BasicTrack () )
{}

std::string Track::tag() const
{
  return data().tag();
}

Charge Track::charge() const
{
  return data().charge();
}

int Track::id() const
{
  return data().id();
}

const Vector6D & Track::state() const
{
  return data().state();
}

const Covariance6D & Track::error() const
{
  return data().error();
}

void * Track::originalObject() const
{
  return data().originalObject();
}

std::vector < Track > Track::components() const
{
  std::vector < Track > ret;
  for ( std::vector< BasicTrack >::const_iterator i=data().components().begin();
        i!=data().components().end() ; ++i )
  {
    ret.push_back ( Track ( *i ) );
  }
  return ret;
}

bool Track::isValid() const
{
  return data().isValid();
}

bool Track::operator< ( const Track & o ) const
{
    return ( id() < o.id() );
}

bool Track::operator== ( const Track & o ) const
{
  return ( id() == o.id() );
}

double Track::chi2() const
{
  return data().chi2();
}

double Track::ndof() const
{
  return data().ndof();
}

